# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'qute_style\ui\whats_new_window.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_whats_new_window(object):
    def setupUi(self, whats_new_window):
        whats_new_window.setObjectName("whats_new_window")
        whats_new_window.resize(389, 341)
        self.centralwidget = QtWidgets.QWidget(whats_new_window)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.scrollArea = QtWidgets.QScrollArea(self.centralwidget)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 369, 249))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(
            self.scrollAreaWidgetContents
        )
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.text_label = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.text_label.setText("")
        self.text_label.setWordWrap(True)
        self.text_label.setObjectName("text_label")
        self.horizontalLayout_3.addWidget(self.text_label)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.verticalLayout.addWidget(self.scrollArea)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(
            40,
            20,
            QtWidgets.QSizePolicy.Expanding,
            QtWidgets.QSizePolicy.Minimum,
        )
        self.horizontalLayout.addItem(spacerItem)
        self.previous_button = QtWidgets.QPushButton(self.centralwidget)
        self.previous_button.setObjectName("previous_button")
        self.horizontalLayout.addWidget(self.previous_button)
        self.next_button = QtWidgets.QPushButton(self.centralwidget)
        self.next_button.setObjectName("next_button")
        self.horizontalLayout.addWidget(self.next_button)
        self.close_button = QtWidgets.QPushButton(self.centralwidget)
        self.close_button.setObjectName("close_button")
        self.horizontalLayout.addWidget(self.close_button)
        self.verticalLayout.addLayout(self.horizontalLayout)
        whats_new_window.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(whats_new_window)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 389, 21))
        self.menubar.setObjectName("menubar")
        whats_new_window.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(whats_new_window)
        self.statusbar.setObjectName("statusbar")
        whats_new_window.setStatusBar(self.statusbar)

        self.retranslateUi(whats_new_window)
        QtCore.QMetaObject.connectSlotsByName(whats_new_window)

    def retranslateUi(self, whats_new_window):
        _translate = QtCore.QCoreApplication.translate
        whats_new_window.setWindowTitle(
            _translate("whats_new_window", "StyledMainWindow")
        )
        self.previous_button.setToolTip(
            _translate("whats_new_window", "Den letzten Eintrag anzeigen.")
        )
        self.previous_button.setText(_translate("whats_new_window", "Zurück"))
        self.next_button.setToolTip(
            _translate("whats_new_window", "Den nächsten Eintrag anzeigen.")
        )
        self.next_button.setText(_translate("whats_new_window", "Weiter"))
        self.close_button.setToolTip(
            _translate(
                "whats_new_window",
                'Das Fenster schließen. Sie können das Fenster erneut anzeigen, indem Sie im Menü "Hilfe" den Eintrag "Neuerung" anwählen.',
            )
        )
        self.close_button.setText(_translate("whats_new_window", "Schließen"))
